﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using SunVoxSharp;

namespace SunVoxPlayer
{
    public partial class FormBasic : Form
    {
        public FormBasic(Icon ico)
        {
            this.Icon = ico;
            InitializeComponent();
        }

        SunVoxCore svCore;
        SunVoxTrack svTrack;
        byte[][] soundeffects;
        bool initCompleted;

        private void FormBasic_Load(object sender, EventArgs e)
        {
            initCompleted = false;
        }
        private void FormBasic_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.P)
            {
                if (initCompleted)
                {
                    MessageBox.Show("you already did that");
                    return;
                }
                label_info.Visible = false;
                Application.DoEvents();
                long ts1 = DateTime.UtcNow.Ticks / 10000;



                /*
                 * The code below is all you need to:
                 *   - Put library in sound-effect preload mode
                 *   - Preload a bunch of sound effects to memory
                 *   - Unload and reload library for streaming BGM
                 * 
                 * The writes to "sunvox_dll_config.ini" are optional,
                 * but makes rendering both faster and more accurate
                 */

                File.WriteAllText("sunvox_dll_config.ini", "buffer 64");
                svCore = new SunVoxCore(false, false);

                svTrack = null;
                soundeffects = new byte[][] {
                    svCore.render("sfx1.sunvox", 0.5),
                    svCore.render("sfx2.sunvox", 0.5),
                    svCore.render("sfx3.sunvox", 0.5)
                };
                // rendering an empty file to clear up memory
                svCore.render("z.sunvox", 0);
                svCore.dispose();
                
                File.WriteAllText("sunvox_dll_config.ini", "buffer 4096");
                svCore = new SunVoxCore(true, true);

                /*
                 * end of sunvox related code
                 */
                


                long ts2 = DateTime.UtcNow.Ticks / 10000;
                initCompleted = true;
                label_info.Visible = true;
                System.Media.SystemSounds.Asterisk.Play();

                this.Text = string.Format(
                    "Loaded libSunVox v{0}.{1}.{2}  (rendering SFX took {3} msec)",
                    svCore.VER_MAJOR,
                    svCore.VER_MINOR,
                    svCore.VER_PATCH,
                    ts2 - ts1);
            }
            if (!initCompleted)
            {
                MessageBox.Show("press P first");
                return;
            }
            int playSFX = -1;
            if (e.KeyCode == Keys.Q) playSFX = 0;
            if (e.KeyCode == Keys.W) playSFX = 1;
            if (e.KeyCode == Keys.E) playSFX = 2;
            if (playSFX >= 0)
            {
                /*
                 * this uses the WinForms SoundPlayer to play the
                 * preloaded .wav files from memory, You'll probably
                 * do this very differently in XNA.
                 */
                using (MemoryStream sfx = new MemoryStream(soundeffects[playSFX]))
                {
                    new System.Media.SoundPlayer(sfx).Play();
                }
            }
            int playBGM = -1;
            if (e.KeyCode == Keys.A) playBGM = 0;
            if (e.KeyCode == Keys.S) playBGM = 1;
            if (e.KeyCode == Keys.D) playBGM = 2;
            if (playBGM >= 0)
            {
                if (svTrack != null)
                {
                    // stop and unload old song
                    svTrack.dispose();
                }

                string filename = "bgm" + (playBGM+1) + ".sunvox";
                
                // loading by filename:
                //svTrack = svCore.load(filename);
                //svTrack.play();

                // loading from memory:
                byte[] data = File.ReadAllBytes(filename);
                svTrack = svCore.load(data);
                svTrack.play();
            }
        }

        private void FormBasic_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (svCore != null)
            {
                svCore.dispose();
            }

        }




















        // this was used to determine the optimal
        // sunvox buffer size for rendering to memory
        void speedtest()
        {
            svTrack = null;
            StringBuilder ret = new StringBuilder();
            for (int a = 8; a < 1024; a += 8)
            {
                ret.AppendLine();
                ret.Append(a);
                File.WriteAllText("sunvox_dll_config.ini", "buffer " + a);
                for (int b = 0; b < 2; b++)
                {
                    ret.Append("     ");
                    svCore = new SunVoxCore(false, false);
                    long t1 = DateTime.UtcNow.Ticks / 10000;
                    soundeffects = new byte[][] {
                            svCore.render("sfx1.sunvox", 0.5),
                            svCore.render("sfx2.sunvox", 0.5),
                            svCore.render("sfx3.sunvox", 0.5)
                        };
                    long t2 = DateTime.UtcNow.Ticks / 10000;
                    svCore.render("z.sunvox", 0);
                    svCore.dispose();
                    this.Text = (a + "    " + b + "    " + (t2 - t1));
                    Application.DoEvents();
                    ret.Append((t2 - t1).ToString() + "   " + soundeffects[0].Length + "," + soundeffects[1].Length + "," + soundeffects[2].Length);
                }
            }
            Clipboard.Clear();
            Clipboard.SetText(ret.ToString());
            svCore = new SunVoxCore(true, true);
        }
    }
}
