﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SunVoxSharp;

namespace SunVoxPlayer
{
    public partial class FormListen : Form
    {
        public FormListen(Icon ico)
        {
            this.Icon = ico;
            InitializeComponent();
        }

        double ampWidth;
        SunVoxCore svCore;
        SunVoxTrack svTrack;
        DiscoLightManager collectors;
        DiscoLightManager eventSockets;

        private void Form1_Load(object sender, EventArgs e)
        {
            // program init from another event so we can see any exceptions
            Timer t = new Timer();
            t.Tick += t_Tick;
            t.Interval = 100;
            t.Start();
        }

        void t_Tick(object sender, EventArgs e)
        {
            ((Timer)sender).Stop();
            svCore = new SunVoxCore(true, false);
            this.Text = string.Format(
                "libSunVox v{0}.{1}.{2}",
                svCore.VER_MAJOR,
                svCore.VER_MINOR,
                svCore.VER_PATCH);
            svTrack = svCore.load("gcc.sunvox");
            svTrack.play();

            // Listen to the modules that we need events from
            svTrack.listen(0x16); // kick (instrument module)
            svTrack.listen(0x15); // snare (instrument module)
            svTrack.listen(0x01); // pluck
            svTrack.listen(0x12); // kick (event module)
            svTrack.listen(0x13); // snare (event module)

            // Prepare stuff for the kickdrum amplitude bar,
            // with its value manually read from the SunVoxTrack
            // using a relatively high resolution timer
            ampWidth = label1.Width;
            Timer tAmp = new Timer();
            tAmp.Tick += tAmp_Tick;
            tAmp.Interval = 10;
            tAmp.Start();





            // Find all labels named collect_* in the form
            // (intended targets for collectEvents),
            // and include them in the DiscoLightManager
            // for timer-based amplitude-event reads
            collectors = new DiscoLightManager(this.Controls, "collect_");

            // create a low resolution timer that
            // periodically collects all queued events
            // from sunvoxtrack and applies them to
            // the collector DiscoLightManager (GUI)
            Timer tCollect = new Timer();
            tCollect.Tick += tCollect_Tick;
            tCollect.Interval = 100;
            tCollect.Start();




            
            // event listener for directly fired events from sunvoxtrack,
            // and a DiscoLightManager to show the amplitudes in the GUI
            eventSockets = new DiscoLightManager(this.Controls, "event_");
            svTrack.ModuleSignalPeak += svTrack_ModuleSignalPeak;





            // add a way to modify the echo module in the song
            delay_inner.MouseDown += new MouseEventHandler(delay_event);
            delay_inner.MouseMove += new MouseEventHandler(delay_event);
            delay_outer.MouseDown += new MouseEventHandler(delay_event);
            delay_outer.MouseMove += new MouseEventHandler(delay_event);





            // and finally a common timer to decay the
            // colors of all disco-labels in the form
            Timer tDecay = new Timer();
            tDecay.Tick += tDecay_Tick;
            tDecay.Interval = 1;
            tDecay.Start();
        }

        // Modify the audio delay
        void delay_event(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left) return;
            double rel = (e.X * 1.0) / delay_outer.Width;
            
            // these values match those displayed in the SunVox GUI
            int mod = 0x14;
            int ctl = 4;

            int param = (int)(0x8000 * rel);
            int value = param / (0x8000 / 256);

            svTrack.sendEvent(0, 0, 0, mod + 1, ctl * 0x100, param);

            // assert that the value was actually written to the
            // echo module by reading it from the module itself
            value = svTrack.getModuleParameter(mod, ctl - 1, 0x8000);
            value = param / (0x8000 / 256);

            delay_inner.Text = value < 128 ? "" : value.ToString();
            delay_outer.Text = value > 128 ? "" : value.ToString();
            delay_inner.Width = e.X;
        }

        // Colour fading timer
        void tDecay_Tick(object sender, EventArgs e)
        {
            collectors.decayColors();
            eventSockets.decayColors();
        }

        // This is called by the SunVoxTrack directly; this method
        // generates a new colour for the GUI label (based on the
        // event amplitude) and updates the GUI without timers/latency
        void svTrack_ModuleSignalPeak(object sender, SignalEvent e)
        {
            eventSockets.setLevel(e.module, e.volume);
        }

        // Instrument used for the topmost amplitude bar
        int instrument =
            //0x04; // sq.slow
            //0x0B; // saw.fast
            //0x08; // d.bass
            //0x0D; // d.snare
              0x12; //#d.bass

        // and the amplitude bar junk itself
        void tAmp_Tick(object sender, EventArgs e)
        {
            if (svTrack == null)
            {
                ((Timer)sender).Stop();
                return;
            }
            double amp = svTrack.getAmplitude(instrument);
            amp = Math.Min(amp, 1);
            int c = (int)(95+160*(1-Math.Pow(amp, 2)));
            label1.Width = (int)(ampWidth * amp);
            label2.BackColor = Color.FromArgb(c, c, c);

            /*HSL hsl = new HSL(
                0.2 * (1 - amp),
                0.5 + 0.5 * amp,
                0.4 + 0.1 * (1 - amp));
            label1.BackColor = (Color)hsl;*/
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (svCore != null)
            {
                svCore.dispose();
            }
            svTrack = null;
            svCore = null;
        }

        // collect signal events from sunvoxtrack and blink the boxes
        void tCollect_Tick(object sender, EventArgs e)
        {
            int events = 0;
            if (svTrack == null) return;
            foreach (SunVoxSharp.SignalEvent se in svTrack.collectEvents())
            {
                events++;
                collectors.setLevel(se.module, se.volume);
            }
            this.Text = "collected: " + events;

            foreach (var unit in collectors.units)
                unit.label.Text = unit.text + "  " + unit.fires;

            foreach (var unit in eventSockets.units)
                unit.label.Text = unit.text + "  " + unit.fires;
        }

        private void label12_Click(object sender, EventArgs e)
        {
            label12.Visible = false;
            label14.Visible = true;
        }

        private void label14_Click(object sender, EventArgs e)
        {
            label14.Visible = false;
            label12.Visible = true;
        }
    }
}
