﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SunVoxSharp;
using System.IO;

namespace SunVoxPlayer
{
    public partial class FormPlayer : Form
    {

        public FormPlayer(Icon ico)
        {
            this.Icon = ico;
            InitializeComponent();
        }

        const double MAX_VOL = 256;

        SunVoxCore svCore;
        SunVoxTrack svTrack;

        private void Form1_Load(object sender, EventArgs e)
        {
            svCore = null;
            svTrack = null;
            
            // program init from another event so we can see any exceptions
            Timer t_init = new Timer();
            t_init.Tick += new EventHandler(t_init_Tick);
            t_init.Interval = 100;
            t_init.Start();

            // playback stats
            Timer t_stats = new Timer();
            t_stats.Tick += new EventHandler(t_stats_Tick);
            t_stats.Interval = 10;
            t_stats.Start();
        }

        // handle user clicking the volume slider
        void vol_event(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left) return;
            double rel = (e.X * 1.0) / vol_outer.Width;
            svTrack.volume = (int)(rel * MAX_VOL);
        }

        // handle user clicking the position slider
        void pos_event(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left) return;
            double rel = (e.X * 1.0) / pos_outer.Width;
            svTrack.position = (int)(svTrack.length * rel);
        }

        // program init
        void t_init_Tick(object sender, EventArgs e)
        {
            ((Timer)sender).Stop();

            /*byte[] riff = System.IO.File.ReadAllBytes("riff");
            string raff = Convert.ToBase64String(riff);
            Clipboard.Clear();
            Clipboard.SetText(raff);
            System.Diagnostics.Process.GetCurrentProcess().Kill();*/

            // hook GUI events
            vol_inner.MouseDown += new MouseEventHandler(vol_event);
            vol_inner.MouseMove += new MouseEventHandler(vol_event);
            vol_outer.MouseDown += new MouseEventHandler(vol_event);
            vol_outer.MouseMove += new MouseEventHandler(vol_event);
            pos_inner.MouseDown += new MouseEventHandler(pos_event);
            pos_inner.MouseMove += new MouseEventHandler(pos_event);
            pos_outer.MouseDown += new MouseEventHandler(pos_event);
            pos_outer.MouseMove += new MouseEventHandler(pos_event);

            // find songs
            string[] files = System.IO.Directory.GetFiles(".");
            Array.Sort(files);
            foreach (string file in files)
            {
                if (file.EndsWith(".sunvox"))
                {
                    songur.Items.Add(file.Substring(2, file.Length - 9));
                }
            }

            // initialize sunvoxcore for regular operation (live playback)
            svCore = new SunVoxCore(true, false);
            
            // debug: display library version in window title
            this.Text = string.Format(
                "libSunVox v{0}.{1}.{2}",
                svCore.VER_MAJOR,
                svCore.VER_MINOR,
                svCore.VER_PATCH);

            // play first track in list
            playTrack(0);
        }

        void playTrack(int i)
        {
            if (svTrack != null)
            {
                svTrack.dispose();
            }

            // load the song...
            svTrack = svCore.load(songur.Items[i] + ".sunvox");

            // ...and play it!
            svTrack.play();

            // might as well update the gui too
            songur.SelectedIndex = i;
        }

        // playback stats
        void t_stats_Tick(object sender, EventArgs e)
        {
            if (svTrack != null)
            {
                int vol = svTrack.volume;
                int len = svTrack.length;
                int pos = svTrack.position;
                double dpos = pos * 1.0 / svTrack.length;

                string text_vol = vol.ToString();
                string text_pos = len + " / " + pos;

                if (vol > MAX_VOL / 2)
                {
                    vol_inner.Text = text_vol;
                    vol_outer.Text = "";
                }
                else
                {
                    vol_inner.Text = "";
                    vol_outer.Text = text_vol;
                }
                if (dpos > 0.5)
                {
                    pos_inner.Text = text_pos;
                    pos_outer.Text = "";
                }
                else
                {
                    pos_inner.Text = "";
                    pos_outer.Text = text_pos;
                }
                pos_perc.Text = Math.Round((pos * 100.0) / svTrack.length, 1) + "%";
                vol_inner.Width = (int)(vol_outer.Width * vol / MAX_VOL);
                pos_inner.Width = (int)(pos_outer.Width * dpos);
            }
        }

        private void songur_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            playTrack(songur.SelectedIndex);
        }

        private void FormPlayer_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (svCore != null)
            {
                svCore.dispose();
            }
            svTrack = null;
            svCore = null;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int i = songur.SelectedIndex - 1;
            if (i < 0) i = songur.Items.Count - 1;
            playTrack(i);
        }

        private void button4_Click(object sender, EventArgs e)
        {
            int i = songur.SelectedIndex + 1;
            if (i > songur.Items.Count - 2) i = 0;
            playTrack(i);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            svTrack.stop();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            svTrack.resume();
        }
    }
}
