﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using SunVoxSharp;

namespace SunVoxPlayer
{
    public partial class FormSplash : Form
    {
        public FormSplash()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Hide();
            new FormBasic(this.Icon).ShowDialog();
            this.Show();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.Hide();
            new FormPlayer(this.Icon).ShowDialog();
            this.Show();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.Hide();
            new FormListen(this.Icon).ShowDialog();
            this.Show();
        }






        /// <summary>
        /// This method, and the "packer" directory, are
        /// unrelated to SunVoxSharp and the SunVox code.
        /// They are used to pack all dependencies
        /// into the main SunVoxPlayer binary.
        /// 
        /// Since this is for convenience purposes only,
        /// I didn't bother keeping the code clean. Sorry
        /// </summary>
        private void FormSplash_Load(object sender, EventArgs e)
        {
            if (Directory.Exists("../../packer/dist"))
            {
                if (DialogResult.Yes == MessageBox.Show(
                    "found ../../packer/dist, make new dfc?", "packer",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question))
                {
                    new DFC().make(null);
                    MessageBox.Show("done");
                    System.Diagnostics.Process.GetCurrentProcess().Kill();
                }
            }
            string lookfor = "SunVoxSharp.dll";
            string self = System.Diagnostics.Process.GetCurrentProcess().ProcessName + ".exe";

            if (File.Exists(lookfor))
                return;

            if (!File.Exists(Program.svs + lookfor))
            {
                //var fu = new packer.FormUnpacking();
                //fu.Show();

                //NotifyIcon ni = new NotifyIcon();
                //ni.Visible = true;
                //ni.Icon = this.Icon;
                //ni.Text = "SunVoxPlayer";
                //ni.ShowBalloonTip(5000, "Extracting", "Please wait just a bit", ToolTipIcon.Info);
                //Application.DoEvents();

                new DFC().extract(null);
                File.Copy(self, Program.svs + self);

                //fu.Hide();
                //fu.Dispose();
            }

            if (File.Exists(Program.svs + self) &&
                Program.argv.Length == 0)
            {
                var proc = new Process();
                proc.StartInfo.FileName = self;
                proc.StartInfo.Arguments = "no_forkbomb_pls";
                proc.StartInfo.WorkingDirectory = Program.svs;
                proc.Start();

                Application.DoEvents();
                System.Threading.Thread.Sleep(1000);
                System.Diagnostics.Process.GetCurrentProcess().Kill();
            }
            else
                MessageBox.Show("Something went wrong trying to extract dependencies\r\n\r\nThis ain't gonna fly bro");
        }






        // sunvox instance for conversion ops
        SunVoxCore svCore;

        // the song conversion queue
        List<string> queue = null;

        /// <summary>
        /// indicate that we allow file drops
        /// </summary>
        private void FormSplash_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        /// <summary>
        /// initiate conversion mode
        /// </summary>
        private void FormSplash_DragDrop(object sender, DragEventArgs e)
        {
            if (queue == null)
            {
                queue = new List<string>();

                // this is the first drag-drop, let's kick the covnersion thread up
                System.Threading.Thread thr = new System.Threading.Thread(new System.Threading.ThreadStart(converter));
                thr.Name = "batch_converter";
                thr.Start();

                // also, timer on the gui thread for graphical updates
                Timer ti = new Timer();
                ti.Interval = 100;
                ti.Start();
                ti.Tick += new EventHandler(ti_Tick);
            }
            checkBox1.Visible = true;
            label1.Visible = false;
            queue.AddRange((string[])
                e.Data.GetData(DataFormats.FileDrop));
        }

        /// <summary>
        /// update GUI with remaining songs and stuff
        /// </summary>
        void ti_Tick(object sender, EventArgs e)
        {
            lock (queue)
            {
                int left = queue.Count;

                if (left <= 0)
                {
                    if (checkBox1.Visible)
                    {
                        label2.Text = "Conversion finished";
                        checkBox1.Visible = false;
                        label1.Visible = true;

                        System.Media.SystemSounds.Exclamation.Play();
                        File.WriteAllText("sunvox_dll_config.ini", "buffer 4096");

                        if (checkBox1.Checked)
                            System.Diagnostics.Process.GetCurrentProcess().Kill();
                    }
                }
                else
                    label2.Text = left + " songs left...   Now converting:\r\n\r\n" + queue[0];
            }
        }

        /// <summary>
        /// the conversion worker
        /// </summary>
        void converter()
        {
            while (!pls_exit)
            {
                int remaining = 0;
                string current = "";
                lock (queue)
                {
                    remaining = queue.Count;

                    if (remaining > 0)
                        current = queue[0];
                }
                if (remaining > 0)
                {
                    if (svCore == null)
                    {
                        File.WriteAllText("sunvox_dll_config.ini", "buffer 64");
                        svCore = new SunVoxCore(false, false);
                    }

                    string wavName = current.Substring(0, current.LastIndexOf('.')) + ".wav";
                    byte[] data = svCore.render(current, 10);
                    File.WriteAllBytes(wavName, data);

                    lock (queue)
                    {
                        queue.RemoveAt(0);
                        if (queue.Count <= 0)
                        {
                            if (File.Exists("z.sunvox"))
                                svCore.render("z.sunvox", 0);

                            svCore.dispose();
                            svCore = null;
                        }
                    }
                }
                System.Threading.Thread.Sleep(10);
            }
        }

        bool pls_exit = false;
        private void FormSplash_FormClosing(object sender, FormClosingEventArgs e)
        {
            pls_exit = true;
        }
    }
}
